/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Peter Kaiser <peter dot kaiser at kit dot edu>
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_UNITS_HAPTIC_SLICE_
#define _ARMARX_ROBOTAPI_UNITS_HAPTIC_SLICE_

#include <RobotAPI/interface/units/UnitInterface.ice>
#include <RobotAPI/interface/core/RobotState.ice>

#include <ArmarXCore/interface/core/UserException.ice>
#include <ArmarXCore/interface/core/BasicTypes.ice>
#include <ArmarXCore/interface/observers/VariantBase.ice>
#include <ArmarXCore/interface/observers/Matrix.ice>
#include <ArmarXCore/interface/observers/Timestamp.ice>
#include <ArmarXCore/interface/observers/ObserverInterface.ice>

module armarx
{
	/**
     * Implements an interface to a HapitcUnit.
     */
    interface HapticUnitInterface extends armarx::SensorActorUnitInterface
    {
    };
	/**
     * Implements an interface to a HapitcUnitListener.
     */
    interface HapticUnitListener
    {
		/**
		* reportSensorValues reports the tactile sensor values from a given sensor device.
		* @param device Name of tactile sensor device.
		* @param values Matrix of taxels.
		* @param timestamp Timestamp of the measurement.
		**/
        void reportSensorValues(string device, string name, MatrixFloatBase values, TimestampBase timestamp);
    };
	/**
     * Implements an interface to a HapitcUnitObserver.
     */
    interface HapticUnitObserverInterface extends ObserverInterface, HapticUnitListener
    {
    };

};

#endif
