/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Martin Miller (martin dot miller at student dot kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#ifndef ARMARX_RobotAPI_FramedPointWithNormal_H
#define ARMARX_RobotAPI_FramedPointWithNormal_H


#include "PointWithNormal.h"

#include <VirtualRobot/Robot.h>
#include <VirtualRobot/LinkedCoordinate.h>
#include <VirtualRobot/LinkedCoordinate.h>
#include <VirtualRobot/VirtualRobot.h>

#include <RobotAPI/interface/core/RobotState.h>
#include "FramedPose.h"

namespace armarx
{
    namespace VariantType
    {
        // variant types
        const VariantTypeId FramedPointWithNormal = Variant::addTypeName("::armarx::FramedPointWithNormalBase");
    }

    class FramedPointWithNormal;
    typedef IceInternal::Handle<FramedPointWithNormal> FramedPointWithNormalPtr;

    class FramedPointWithNormal :
        virtual public FramedPointWithNormalBase,
        virtual public PointWithNormal
    {
    public:
        FramedPointWithNormal();
        FramedPointWithNormal(const FramedPointWithNormal& source);
        FramedPointWithNormal(const Eigen::Vector3f& position, const Eigen::Vector3f& normal, const std::string& frame, const std::string& agent);
        FramedPointWithNormal(const PointWithNormal& pointWithNormal, const std::string& frame, const std::string& agent);
        FramedPointWithNormal(Ice::Float px, ::Ice::Float py, ::Ice::Float pz, Ice::Float nx, ::Ice::Float ny, ::Ice::Float nz, const std::string& frame, const std::string& agent);

        std::string getFrame() const;
        void changeFrame(const VirtualRobot::RobotPtr& robot, const std::string& newFrame);

        void changeToGlobal(const SharedRobotInterfacePrx& referenceRobot);
        void changeToGlobal(const VirtualRobot::RobotPtr& referenceRobot);
        FramedPointWithNormalPtr toGlobal(const SharedRobotInterfacePrx& referenceRobot) const;
        FramedPointWithNormalPtr toGlobal(const VirtualRobot::RobotPtr& referenceRobot) const;
        Eigen::Vector3f positionToGlobalEigen(const SharedRobotInterfacePrx& referenceRobot) const;
        Eigen::Vector3f normalToGlobalEigen(const SharedRobotInterfacePrx& referenceRobot) const;
        Eigen::Vector3f positionToGlobalEigen(const VirtualRobot::RobotPtr& referenceRobot) const;
        Eigen::Vector3f normalToGlobalEigen(const VirtualRobot::RobotPtr& referenceRobot) const;
        FramedPointWithNormalPtr toRootFrame(const SharedRobotInterfacePrx& referenceRobot) const;
        FramedPointWithNormalPtr toRootFrame(const VirtualRobot::RobotPtr& referenceRobot) const;
        Eigen::Vector3f positionToRootEigen(const SharedRobotInterfacePrx& referenceRobot) const;
        Eigen::Vector3f normalToRootEigen(const SharedRobotInterfacePrx& referenceRobot) const;
        Eigen::Vector3f positionToRootEigen(const VirtualRobot::RobotPtr& referenceRobot) const;
        Eigen::Vector3f normalToRootEigen(const VirtualRobot::RobotPtr& referenceRobot) const;

        FramedPosition getFramedPosition() const;
        FramedDirection getFramedNormal() const;

        // inherited from VariantDataClass
        Ice::ObjectPtr ice_clone() const
        {
            return this->clone();
        }
        VariantDataClassPtr clone(const Ice::Current& c = ::Ice::Current()) const
        {
            return new FramedPointWithNormal(*this);
        }
        std::string output(const Ice::Current& c = ::Ice::Current()) const;
        VariantTypeId getType(const Ice::Current& c = ::Ice::Current()) const
        {
            return VariantType::FramedPointWithNormal;
        }
        bool validate(const Ice::Current& c = ::Ice::Current())
        {
            return true;
        }

        friend std::ostream& operator<<(std::ostream& stream, const FramedPointWithNormal& rhs)
        {
            stream << "FramedPointWithNormal: " << std::endl << rhs.output() << std::endl;
            return stream;
        }

    public: // serialization
        virtual void serialize(const armarx::ObjectSerializerBasePtr& serializer, const ::Ice::Current& = ::Ice::Current()) const;
        virtual void deserialize(const armarx::ObjectSerializerBasePtr& serializer, const ::Ice::Current& = ::Ice::Current());

    };
}

#endif // armarx_core_FramedPointWithNormal
