/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Martin Miller (martin dot miller at student dot kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#ifndef ARMARX_RobotAPI_PointWithNormal_H
#define ARMARX_RobotAPI_PointWithNormal_H


#include <RobotAPI/interface/core/PointWithNormal.h>
#include <ArmarXCore/observers/variant/Variant.h>
#include <ArmarXCore/observers/AbstractObjectSerializer.h>

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <sstream>

namespace armarx
{
    namespace VariantType
    {
        // variant types
        const VariantTypeId PointWithNormal = Variant::addTypeName("::armarx::PointWithNormalBase");
    }

    class PointWithNormal:
        public virtual PointWithNormalBase
    {

    public:
        PointWithNormal();
        PointWithNormal(const Eigen::Vector3f& position, const Eigen::Vector3f& normal);
        PointWithNormal(::Ice::Float px, ::Ice::Float py, ::Ice::Float pz, ::Ice::Float nx, ::Ice::Float ny, ::Ice::Float nz);

        virtual Eigen::Vector3f positionToEigen() const;
        virtual Eigen::Vector3f normalToEigen() const;

        // inherited from VariantDataClass
        Ice::ObjectPtr ice_clone() const
        {
            return this->clone();
        }
        VariantDataClassPtr clone(const Ice::Current& c = ::Ice::Current()) const
        {
            return new PointWithNormal(*this);
        }
        std::string output(const Ice::Current& c = ::Ice::Current()) const;
        VariantTypeId getType(const Ice::Current& c = ::Ice::Current()) const
        {
            return VariantType::PointWithNormal;
        }
        bool validate(const Ice::Current& c = ::Ice::Current())
        {
            return true;
        }

        friend std::ostream& operator<<(std::ostream& stream, const PointWithNormal& rhs)
        {
            stream << "PointWithNormal: " << std::endl << rhs.output() << std::endl;
            return stream;
        }

    public: // serialization
        virtual void serialize(const armarx::ObjectSerializerBasePtr& serializer, const ::Ice::Current& = ::Ice::Current()) const;
        virtual void deserialize(const armarx::ObjectSerializerBasePtr& serializer, const ::Ice::Current& = ::Ice::Current());
    };
}

#endif // armarx_core_PointWithNormal
